'****************************************************************
'*  Name    : RSOut8Digits_DemuV11_Lib.inc                      *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 04/07/2018                                        *
'*  Version : 1.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.0.3                    *
'*          : Original Code written by Les Johnson              *
'* Purpose  : Library for Multiplexing 2 to 8 digits LED display*
'* Hardware : with 1 PORT + multiplexer to reduce pins count    *
'*          : 74HC238 + UDN2981A or ULN2803                     *
'* Software : Using the modified RsOut command                  *
'****************************************************************
'/ 02/07/2018 version 1.1
'/ In case of ASCII characters > 123 goto error.
'/ Moved the Dot and Minus detection in Select/Case code.
'/ Modified the mode to write the PORT_Digits. Saving 8 lines of code.
'/ Modified the Data Table and detection code, save more than 20 bytes of code.

(*
Recommended identifiers for the RsOut command.
The modifiers are listed below:
Modifier Operation
    Bin{1..To number of digits} Send binary digits
    Dec{0..To number of digits} Send decimal digits (amount of digits after decimal point with floating point)
    Hex{1..To number of digits} Send hexadecimal digits
    SBin{1..To number of digits} Send signed binary digits
    SDec{0..limited} Send signed decimal digits
    SHex{1..To number of digits} Send signed hexadecimal digits

Mode:
Examples:
    RsOut Dec MyWord,CR
    RsOut Dec3 MyFloat,CR
    RsOut HEX3 MyWord,CR
    RsOut Bin4 MyWord,CR 

Or to avoid tipo errata, use the Macros.
    PrintDispInt(Dec, MyDword)
    PrintDispInt(Bin8, MyWord)  
    PrintDispFlt(Dec3, MyFloat)    
    PrintDispSign(SDec, MySDword)
    PrintDispSign(Bin7, MySDword)      

It is possible to print some ASCII characters like this (use low case characters only).
    MyASCIIarray = "STOP"
    RsOut Str MyASCIIarray\4,CR

    MyASCIIarray = "INPUT"
    RsOut Str MyASCIIarray\5,CR

    MyASCIIarray = "OUPUT"
    RsOut Str MyASCIIarray\6,CR

Note: The ":",";","<","=",">","?","@" mean a SPACE in the display.  
*)
'/------------------------------------------------------------------------------
$ifndef _MultiplexDigits
$define _MultiplexDigits

'/------------------------------------------------------------------------------
    $if _device != _18F452
    $error "This library is for the PIC18F452 device only. Please write a new library"
    $endif

    $ifdef _ProteusTest
        $if _xtal != 8
        $error "It is better to use the Declare Xtal = 8 for Proteus"
        $endif   
    $endif   
'/==============================================================================
'/ Configuration of the Config Fuses
    $define PLL_ConfigFuses OFF
    '$define _CLKOUT_Function_

'/------------------------------------------------------------------------------
'/ Configuration of the displays hardware.
    '/ Declare Display Mode (0 => Common Cathode) (1 => Common Anode) with transistors
    $define DisplayMode 0    

    '/ Declare Number of Digits (3,4,5,6,7,8)
    $define NumberDigits 8

    '/ Declare the Delay to see the Error in the Display.
    '/ The true delay is [_ErrorDelay X Timing(mS)]
    $define _ErrorDelay 500

'/==============================================================================
'/==============================================================================
'/ Declare the Timer used for scanning. (one only)
'    $define ScanTimer0
    $define ScanTimer2

'/------------------------------------------------------------------------------
    '/ To see the value in the asm file.
    #define __ErrorDelay _ErrorDelay    

'/ Declare the interruption flags (automatic).
    $ifdef ScanTimer0
    $define DisPlayInterruptFlag INTCONBits_TMR0IF
    $define DisPlayInterruptEnable INTCONBits_TMR0IE
    #define __INTERRUPT_TIMER0 1    '/ To check the TIMER0  in the ASM file.
    $endif

    $ifdef ScanTimer2    
    $define DisPlayInterruptFlag PIR1Bits_TMR2IF
    $define DisPlayInterruptEnable PIE1Bits_TMR2IE
    #define __INTERRUPT_TIMER2 1    '/ To check the TIMER2 in the ASM file.
    $endif

'/ Declare Hardware type (8 displays with 1 PORT + demultiplexer)
    $if DisplayMode = 0
    $define _Hardware 0 '/ Common Cathode: 74HC238 + UDL2803  
    $else
    $define _Hardware 1 '/ Common Anode: 74HC238 + UDN2981A
    $endif 
'===============================================================================
'/ Declare the Carriage Return
    $ifndef CR
    $define CR 13
    $endif
'/ Declare the Line Feed
    $ifndef LF
    $define LF 10
    $endif

'/ Calculate the better option for Table Data (automaticly).
    '/ Declare Table Data (0 from ROM, 1 from eeprom)
    $if _eeprom > 0
    $define TableData 1    
    #define __TableDataEeprom 1 '/ To check the Table Mode in the ASM file.
    $else
    $define TableData 0    
    #define __TableDataROM 1    '/ To check the Table Mode in the ASM file.
    $endif

    $if _Hardware = 0 
    #define __HARDWARE1PORT_UDN2981A 1  '/ To check the Hardware in the ASM file.
    $else
    #define __HARDWARE1PORT_ULN2803 1   '/ To check the Hardware in the ASM file.
    $endif
                     
'/ Declare the outputs of scanning.
    $if DisplayMode = 0
        $define TurnOffDisp 0                   '/ Turn Off For transistor NPN    
        $define TurnOnDisp 1                    '/ Turn On For transistor NPN
        #define __DisplayModeCommonCathode 1    '/ To check the Display Mode in the ASM file.
    $else
        $define TurnOffDisp 1                   '/ Turn Off For transistor PNP    
        $define TurnOnDisp 0                    '/ Turn On For transistor PNP  
        #define __DisplayModeCommonAnode 1      '/ To check the Display Mode in the Asm file.
    $endif 

    $if NumberDigits < 3 Or NumberDigits > 8
    $error "NO such digits number allowed"
    $endif
    $if DisplayMode > 1
    $error "The DisplayMode must be 0 or 1"
    $endif
    $if TableData > 1
    $error "The Table of Data must be 0 = ROM or 1 = EEPROM"
    $endif

'/ Check a timer
    $if !_defined(ScanTimer0) And !_defined(ScanTimer2)
    $error "Timer0 or Timer2 must be declared"    
    $endif    

'/ Configuration of the RsOut Serial comunication
    Declare RsOut_Pin PORTD.0       '/ It is not a real pin.
    Declare RsOut_Mode Inverted   
    Declare Serial_Baud 9600    
'    Declare Serial_Data 8
'    Declare RsOut_Pace 5
'    Declare Shift_DelayUs 2
'===============================================================================
    GoTo _MultiplexDigitsExit
'===============================================================================
'/**** Added by Fuse Configurator ****
'/ Use the Fuse Configurator plug-in to change these settings

Config_Start
$if PLL_ConfigFuses = On
    $if (_xtal >= 16) And (_xtal <= 40)
    OSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC)(Xtal piece 2,5-10 MHz)
    $else
        $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $if (_xtal >= 4) And (_xtal <= 25)
    OSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10-32 MHz)
    $else
        $if (_xtal > 25) And (_xtal <= 40)
            $ifdef _CLKOUT_Function_
            OSC = EC     ; Exterior EC oscillator, CLKOUT function On RA6 
            $else
            OSC = ECIO   ; Exterior EC oscillator, port function on RA6 
            $endif
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
        $endif
   $endif
$endif
    OSCS = OFF	;Oscillator system clock switch option is disabled (main oscillator is source)
    PWRT = On	;PWRT enabled
    BOR = On	;Brown-out Reset enabled
    BORV = 27	;VBOR set to 2.7V
    WDT = OFF	;WDT disabled (control is placed on the SWDTEN bit)
    WDTPS = 128	;1:128
    CCP2MUX = On	;CCP2 input/output is multiplexed with RC1
    STVR = OFF	;Stack Full/Underflow will not cause RESET
    LVP = OFF	;Low Voltage ICSP disabled
    Debug = OFF	;Background Debugger disabled. RB6 and RB7 configured as general purpose I/O pins.
    Cp0 = OFF	;Block 0 (000200-001FFFh) not code protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code protected
    CPB = OFF	;Boot Block (000000-0001FFh) not code protected
    CPD = OFF	;Data EEPROM not code protected
    WRT0 = OFF	;Block 0 (000200-001FFFh) not write protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write protected
    WRTC = OFF	;Configuration registers (300000-3000FFh) not write protected
    WRTB = OFF	;Boot Block (000000-0001FFh) not write protected
    WRTD = OFF	;Data EEPROM not write protected
    EBTR0 = OFF	;Block 0 (000200-001FFFh) not protected from Table Reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from Table Reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from Table Reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from Table Reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0001FFh) not protected from Table Reads executed in other blocks
Config_End
'/**** End of Fuse Configurator Settings ****
'===============================================================================
    On_Hardware_Interrupt GoTo MULTILEX_DIGITS		' Point to the interrupt handler and ENABLE global interrupts
'===============================================================================       
'/ Using system variables the routines could run faster.
'/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
'/ because too much bytes received.
$if NumberDigits = 8
    #define __NumberDigits 8                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 9
    #define __LenghtDISP_ARRAY 9            '/ Assembler definition. For testing the variables.

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2
    Dim THOUSANDS As DISP_ARRAY#3
    Dim TENTHOUSANDS As DISP_ARRAY#4
    Dim HUNDREDTHOUSANDS As DISP_ARRAY#5
    Dim THOUSANDTHOUSANDS As DISP_ARRAY#6
    Dim TENTHDTHOUSANDS As DISP_ARRAY#7
    
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
$endif

$if NumberDigits = 7
    #define __NumberDigits 7                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 8
    #define __LenghtDISP_ARRAY 8            '/ Assembler definition. For testing the variables.

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2
    Dim THOUSANDS As DISP_ARRAY#3
    Dim TENTHOUSANDS As DISP_ARRAY#4
    Dim HUNDREDTHOUSANDS As DISP_ARRAY#5
    Dim THOUSANDTHOUSANDS As DISP_ARRAY#6
    
    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
$endif

$if NumberDigits = 6
    #define __NumberDigits 6                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 7
    #define __LenghtDISP_ARRAY 7            '/ Assembler definition. For testing the variables.

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2
    Dim THOUSANDS As DISP_ARRAY#3
    Dim TENTHOUSANDS As DISP_ARRAY#4
    Dim HUNDREDTHOUSANDS As DISP_ARRAY#5

    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
$endif

$if NumberDigits = 5
    #define __NumberDigits 5                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 6
    #define __LenghtDISP_ARRAY 6            '/ Assembler definition. For testing the variables.

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2
    Dim THOUSANDS As DISP_ARRAY#3
    Dim TENTHOUSANDS As DISP_ARRAY#4
    
    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
$endif

$if NumberDigits = 4
    #define __NumberDigits 4                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 5
    #define __LenghtDISP_ARRAY 5            '/ Assembler definition. For testing the variables.

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2
    Dim THOUSANDS As DISP_ARRAY#3

    Dim PRINT_ONES As Byte System        '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System        '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System    '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System   '/ Variable for interrupt routine.
$endif
    
$if NumberDigits = 3
    #define __NumberDigits 3                '/ Assembler definition. For testing the variables.
    $define LenghtDISP_ARRAY 4
    #define __LenghtDISP_ARRAY 4            '/ Assembler definition. For testing the variables..

    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim DispDummyArray[10] As Byte      '/ Create a Dummy Array after the DISP_ARRAY to avoid crashing the PIC
    Dim ONES As DISP_ARRAY#0
    Dim TENS As DISP_ARRAY#1
    Dim HUNDREDS As DISP_ARRAY#2

    Dim PRINT_ONES As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System
    Dim PRINT_HUNDREDS As Byte System
$endif
    
'/TIMESHARE depends of number of bytes display.
    $if NumberDigits = 8
    Dim TIMESHARE As Word System    '/ Variable Used by the interrupt for time sharing
    $else
    Dim TIMESHARE As Byte System	'/ Variable Used by the interrupt for time sharing    
    $endif

$ifndef FSR0_Var
    $define FSR0_Var
	Dim FSR0 As FSR0L.Word         'Combine FSR0L\H into a single 16-bit register
$endif
$ifndef FSR1_Var
    $define FSR1_Var
	Dim FSR1 As FSR1L.Word         'Combine FSR1L\H into a single 16-bit register
$endif

'/ Global variables
    Dim dHighPointerFSR1 As Word System
    Dim dRsOutByteIn As Byte System 
    Dim dBytesReceived As Byte System 
    Dim dNumberBytesToShift As Byte System
    Dim dPrintErrorDelay As Word System
   
'/ Flags
    Dim dDISP_Flags As Byte System
    Dim F_PrintErrorDelay As dDISP_Flags.0 

'/ Temporal variables

'/------------------------------------------------------------------------------
'/ Some special constants:
    $define MinusCharacter 191    '/%10111111 (-) value for Common Anode mode.

'/------------------------------------------------------------------------------
    '/ Definition of the PORTS for Display segments.
    Symbol SEGMENTS = LATD
    Symbol PORT_Digits = LATC

    '/ Definition of the pins to enable displays to write.
    Symbol AddressDA = LATC.0           '/ Address 0 of the Digits.
    Symbol AddressDB = LATC.1           '/ Address 1 of the Digits.
    Symbol AddressDC = LATC.2           '/ Address 2 of the Digits.
    Symbol DipsAddressMask = %11110000  '/ LE2 = 0 + Valor to mask all bits of digits addresses.
    Symbol LE2 = LATC.3                 '/ Enable/Disable all Digits. 
   
'===============================================================================
'/ Seven Segment data written for Commom Anode display.
'/ Using some tricks it is not necessary using another table for common cathode.
'/
$if TableData = 0
'/                       0   1   2   3   4   5   6   7   8   9   blank   blank   blank  blank      
SegmentsData: CData Byte 192,249,164,176,153,146,131,248,128,152,255,255,255,255,255,255,255,_
'/A  B   C   D   E   F   G   H   I   J   K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z   [   
136,131,198,161,134,142,144,137,207,225,138,199,200,171,163,140,148,175,146,135,227,193,213,137,145,164,255 '/ 44 numbers
'/ "[" is the character for Space on the display.
'/------------------------------------------------------------------------------
$else
'/------------------------------------------------------------------------------
'/ For more efficiency the Table EData must be placed in address 0 of the eeprom.
'/                       0   1   2   3   4   5   6   7   8   9  blank   blank   blank  blank
SegmentsData EData Byte 192,249,164,176,153,146,131,248,128,152,255,255,255,255,255,255,255,_
'/A  B   C   D   E   F   G   H   I   J   K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z   [   
136,131,198,161,134,142,144,137,207,225,138,199,200,171,163,140,148,175,146,135,227,193,213,137,145,164,255 '/ 44 numbers
'/ "[" is the character for Space on the display.
$endif
'===============================================================================
'===============================================================================
'/ INTERRUPT HANDLER 
'/ Multiplexing the 8-digits
'/ Version 1.1. reducing code, 1 line for every Digit.
'/
MULTILEX_DIGITS:
Context Save				

    If DisPlayInterruptFlag = 1 Then 		
        '/ Process the first display
        If TIMESHARE.0 = 1 Then             '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF.  
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_ONES          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_ONES           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            PORT_Digits = PORT_Digits & DipsAddressMask  '/ Read + clear all bits of Digits address + Multiplexer On.
                                                         '/ Write DIGIT number 1
        '/ Process the second display									
        ElseIf TIMESHARE.1 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENS          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENS           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read + Clear all bits of Digits address + Multiplexer On.
            PORT_Digits = WREG | 1                '/ Write DIGIT number 2

        '/ Process the third display									
        ElseIf TIMESHARE.2 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDS      '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_HUNDREDS       '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 2                '/ Write DIGIT number 3
                
        '/ Process the fourth display			
        $if NumberDigits > 3
        ElseIf TIMESHARE.3 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDS     '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_THOUSANDS      '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 3                '/ Write DIGIT number 4
        $endif

        '/ Process the fifht display			
        $if NumberDigits > 4
        ElseIf TIMESHARE.4 = 1 Then         '/ If it's our turn then...
            LE2 = 1                         '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask  '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 4                '/ Write DIGIT number 5
        $endif

        '/ Process the sixfht display			
        $if NumberDigits > 5
        ElseIf TIMESHARE.5 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_HUNDREDTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 5                  '/ Write DIGIT number 6
        $endif

        '/ Process the sevenfht display			
        $if NumberDigits > 6
        ElseIf TIMESHARE.6 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDTHOUSANDS '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_THOUSANDTHOUSANDS  '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 6                  '/ Write DIGIT number 7
        $endif

        '/ Process the eightfht display			
        $if NumberDigits > 7
        ElseIf TIMESHARE.7 = 1 Then             '/ If it's our turn then...
            LE2 = 1                             '/ All digits OFF. 
            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHDTHOUSANDS   '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENTHDTHOUSANDS    '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            WREG = PORT_Digits & DipsAddressMask    '/ Read & Clear all bits of Digits address.
            PORT_Digits = WREG | 7                  '/ Write DIGIT number 8
        $endif

        EndIf
    
        '/ Process the scanning.
        TIMESHARE = TIMESHARE << 1      '/ Shift bit to set the next digit display.

        '/ Checking the last digit & reset scanning.        
        $if NumberDigits = 8
        If TIMESHARE.8 = 1 Then			'/ Is Bit 8 Set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 7
        If TIMESHARE.7 = 1 Then			'/ Is bit 7 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 6
        If TIMESHARE.6 = 1 Then			'/ Is bit 6 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 5
        If TIMESHARE.5 = 1 Then			'/ Is bit 5 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 4
        If TIMESHARE.4 = 1 Then			'/ Is bit 4 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 3
        If TIMESHARE.3 = 1 Then			'/ Is bit 3 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif        
        DisPlayInterruptFlag = 0        '/ Reset TMR0 interrupt flag

        '/ Security, decrement an error delay
        If F_PrintErrorDelay = 1 Then
            Dec dPrintErrorDelay
            If dPrintErrorDelay = 0 Then
                F_PrintErrorDelay = 0
            EndIf 
        EndIf
    EndIf
Context Restore					        '/ Exit the interrupt routine		

'===============================================================================
'/ Declare some macros for Maths. (To avoid errata errors)
'/ Print all value types. There is not any control of the variable type.
$define PrintDisp(pmodifier, pValue) '
    RsOut pmodifier pValue,CR

'/------------------------------------------------------------------------------
'/ Print Integer values. There is a control of the Variable type.
$define PrintDispInt(pmodifier,pValue) '
    RsOut pmodifier pValue,CR          '
    _mPrintDispInt pValue

'/ This macro is used for testing variables only.
_mPrintDispInt Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispInt - This macro uses 1 parameter!"
    #else
        #if (Prm_1 != Dword) && (Prm_1 != Word) && (Prm_1 != Byte)
            #error "_mPrintDispInt Macro - (Param1) Should be a Dword, Word or Byte Variable"
        #endif
    #endif
Endm

'/------------------------------------------------------------------------------
'/ Print a Signed variable. There is a control of the Variable type.
$define PrintDispSign(pmodifier,pValue) '
    RsOut pmodifier pValue,CR           '
    _mPrintDispSign pValue

'/ This macro is used for testing variables only.
_mPrintDispSign Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispSign - This Macro uses 1 parameter!"
    #else
        #if (Prm_1 != SDword) && (Prm_1 != SWord) && (Prm_1 != SByte)
            #error "_mPrintDispSign Macro - (Param1) Should be a Signed SDword,  SWord or SByte Variable"
        #endif
    #endif
Endm

'/------------------------------------------------------------------------------
'/ Print a Float variable.  There is a control of the Variable type.
$define PrintDispFlt(pmodifier,pValue) '
    RsOut pmodifier pValue,CR          '
    _mPrintDispFloat pValue

'/ This macro is used for testing variables only.
_mPrintDispFloat Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispFloat - This Macro uses 1 parameter!"
    #else
        #if(Prm_1 != Float)
            #error "_mPrintDispFloat Macro - (Param1) Should be a Float Variable"
        #endif
    #endif
Endm

'===============================================================================
'===============================================================================		
'/ Build up the seperate digits of input variable.

    #disable RsOut      '/ Disable the RsOut command to be changed for a code
                        '/ to drive the 8 LEDs displays.
'/==============================================================================
'/ New RsOut Routine.
'/ The array dimension must to be [Number of Digits + 1] to allow the shift of the bytes. 
    Asm
    RsOut
    EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif
    If F_PrintErrorDelay = 1 Then
        Return                      '/ If error return to the RsOut command.
    EndIf
    INDF1 = WREG                    '/ Put the value into the array.
    dRsOutByteIn = WREG             '/ Save it.
    Inc dBytesReceived              '/ Increments Number of bytes received.

    $if TableData = 1
    EECON1 = 0                      '/ Eeprom to read. (only once is sufficient)
    $endif

    '/ Version 1.1. With a new data table distribution, more than 20 lines of code 
    '/ can be saved.
    '/ Checking the higher values first.
    Select dRsOutByteIn
        Case > 90   '/ Looking for no allowed characters values.
            '/ NO increment the pointer. The value is not valid. Goto Error only.
            GoTo SetAnErrorDisp
 
        Case > 47   '/ Looking for decimals variables, Hex variables an High Case ASCII characters.
            $if TableData = 0
                '/ Data from the ROM memory.
                WREG = dRsOutByteIn - 48            '/ Convert ASCIIdec to BCD.
                INDF1 = CRead8 SegmentsData[WREG]   '/ Data for segments.
            $else
                '/ Data from the eeprom memory.
                EEADR = dRsOutByteIn - 48   '/ Convert ASCIIdec to BCD.
                EECON1bits_RD = 1           '/ Set the eeprom to read.
                INDF1 = EEDATA              '/ Data for segments.
            $endif 
            Dec FSR1                        '/ Next array address to write.

        Case 46    '/ Looking for Dot ("."). (Floating DP)
            Inc FSR1                '/ Looking for the last valid digit received.
            INDF1.7 = 0             '/ Set the DP as for common Anode Display in the last digit received. 
            Dec FSR1                '/ Restore the FSR1 pointer for the next byte to receive.
            Dec dBytesReceived      '/ It is not a new value.

        Case 45    '/ Looking for Minus ("-").
            INDF1 = MinusCharacter  '/ Load directly the display value. Less lines of code.
            Dec FSR1                '/ Next array address to write.

        Case 13     '/ Test final bytes; looking For Carriage Return.
            INDF1 = 255                     '/ Clear the CR characters and blanck the position.   
            Dec dBytesReceived              '/ It is not a valid byte.
            GoTo CheckErrorBytesReceived    '/ All bytes received already.
    EndSelect
    Return                          '/ Come back for more bytes; The RsOut commant is in use.
'/------------------------------------------------------------------------------
CheckErrorBytesReceived:
'/ Check the number of bytes received. If YES then print "Error" in the display.
    If dBytesReceived > LenghtDISP_ARRAY - 1 Then
SetAnErrorDisp:
        DisPlayInterruptEnable = 0      '/ Avoid blinking.
        $if NumberDigits = 8
        PRINT_ONES = 175                '/ Character r
        PRINT_TENS = 163                '/ Character o
        PRINT_HUNDREDS = 175            '/ Character r
        PRINT_THOUSANDS = 175           '/ Character r
        PRINT_TENTHOUSANDS = 134        '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        PRINT_THOUSANDTHOUSANDS = 255
        PRINT_TENTHDTHOUSANDS = 255
        $endif
        $if NumberDigits = 7
        PRINT_ONES = 175                '/ Character r
        PRINT_TENS = 163                '/ Character o
        PRINT_HUNDREDS = 175            '/ Character r
        PRINT_THOUSANDS = 175           '/ Character r
        PRINT_TENTHOUSANDS = 134        '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        PRINT_THOUSANDTHOUSANDS = 255
        $endif
        $if NumberDigits = 6
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 163               '/ Character o
        PRINT_HUNDREDS = 175           '/ Character r
        PRINT_THOUSANDS = 175          '/ Character r
        PRINT_TENTHOUSANDS = 134       '/ Character E
        PRINT_HUNDREDTHOUSANDS = 255 
        $endif
        $if NumberDigits = 5
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 163               '/ Character o
        PRINT_HUNDREDS = 175           '/ Character r
        PRINT_THOUSANDS = 175          '/ Character r
        PRINT_TENTHOUSANDS = 134       '/ Character E
        $endif
        $if NumberDigits = 4
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 175               '/ Character r
        PRINT_HUNDREDS = 134           '/ Character E
        PRINT_THOUSANDS = 255  
        $endif 
        $if NumberDigits = 3
        PRINT_ONES = 175               '/ Character r
        PRINT_TENS = 175               '/ Character r
        PRINT_HUNDREDS = 134           '/ Character E
        $endif 
        DisPlayInterruptEnable = 1  
        dPrintErrorDelay = _ErrorDelay  '/ The true delay is [_ErrorDelay X Timing(mS)]
        F_PrintErrorDelay = 1           '/ Flag for interrupt routine.
        GoTo RestoreArrayParameters_Sub
    EndIf
'/------------------------------------------------------------------------------
MoveDatasDispArray:
'/ Prepare the array & move bytes from high position to lower position. (LSB first)
    Dec dBytesReceived               '/ prepare FSR1
    FSR1 = dHighPointerFSR1          '/ Restore the FSR1 to higher byte of the array.
    FSR1 = FSR1 - dBytesReceived     '/ Point the FSR1 at the last byte received.
    FSR0 = VarPtr (DISP_ARRAY)       '/ Point the FSR0 to lower byte of the array.
    Inc dBytesReceived               '/ Restore the correct value of dBytesReceived.
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        INDF0 = INDF1                '/ Copy all bytes
        INDF1 = 255                  '/ Blanking old bytes
        Inc FSR0
        Inc FSR1
        Dec dBytesReceived
    Until dBytesReceived = 0         '/ Shift all the digits already.
'/------------------------------------------------------------------------------
CopyArrayToDisplay_Sub:
    ' Make a copy to print the values for the interrupt routine.
    DisPlayInterruptEnable = 0          '/ Avoid blinking.
    $if NumberDigits = 8
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
    PRINT_TENTHDTHOUSANDS = TENTHDTHOUSANDS
    $endif
    $if NumberDigits = 7
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
    $endif
    $if NumberDigits = 6
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
    $endif            
    $if NumberDigits = 5
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    PRINT_TENTHOUSANDS = TENTHOUSANDS 
    $endif              
    $if NumberDigits = 4
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    PRINT_THOUSANDS = THOUSANDS        
    $endif 
    $if NumberDigits = 3
    PRINT_ONES = ONES
    PRINT_TENS = TENS
    PRINT_HUNDREDS = HUNDREDS
    $endif
    DisPlayInterruptEnable = 1 
'/------------------------------------------------------------------------------
'/ Restore all parameters before to use the RsOut command again.
RestoreArrayParameters_Sub:
'/ Load all the array with blank characters.
'    Clear DISP_ARRAY, blanking all the digits.
    FSR0 = VarPtr (DISP_ARRAY)
    PRODH = LenghtDISP_ARRAY
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTINC0 = 255
        Dec PRODH
    Until PRODH = 0
    DISP_ARRAY[LenghtDISP_ARRAY] = 0    '/ Set Array as String again.

'/ Configure the PORTs again in case of...
	Output SEGMENTS             '/ Put the display PORT as Output.

    '/ Every individual pin is set as output.
    Output AddressDA
    Output AddressDB
    Output AddressDC
    Output LE2

'/ Clear some variables.
    dBytesReceived = 0

'/ Configure the array pointers To start a new RsOut command.
    FSR0 = VarPtr (DISP_ARRAY)              '/ FSR0 At the beginning of the array.
    FSR1 = FSR0 + (LenghtDISP_ARRAY - 1)    '/ FSR1 at the end of the array to start a new RsOut command.
    dHighPointerFSR1 = FSR1                 '/ Make a copy of FRS1.
    Return                                  '/ End of routine RsOut.
'===============================================================================
'===============================================================================
'/ Subroutine: Init the displays
'/ The PDS user must rewrite & check this routine for a new PIC.

$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER0
'/ Set TMR0 to interrupt on rollover
'/ The TMR0H & TMR0l counters are not loaded to simplify the code. Then the timing
'/ could change in every Declare Xtal.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog
    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
    INTCONBits_TMR0IE = 0       '/ Disable the Timer0 interrupts.
    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable priority levels on interrupts.
'/------------------------------------------------------------------------------
'/ Code to configure the Timer0.
$ifdef ScanTimer0
    INTCON2Bits_TMR0IP = 1      '/ Timer0 for High Interrupt.

    '/ Configure the Timer0 for FOSC = 4Mhz.
    $if _xtal = 4
    T0CON = %00000100           '/ div32 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 8Mhz used for Proteus.
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
        T0CON = %00000100           '/ div32 => 4,0 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif
    $else
        $if _xtal = 8
        T0CON = %00000101           '/ div64 => 8,1 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif   
    $endif

    '/ Configure the Timer0 for FOSC = 16Mhz.
    $if _xtal = 16
    T0CON = %00000110           '/ div128 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 20Mhz.
    $if _xtal = 20
    T0CON = %00000110           '/ div128 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 32Mhz.
    $if _xtal = 32
    T0CON = %00000111           '/ div256 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 40Mhz.
    $if _xtal = 40
    T0CON = %00000000           '/ div2 => 13,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 64Mhz for Kxx PICs.
    $if _xtal = 64
    T0CON = %00000001           '/ div4 => 8,2 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 80Mhz for K20 Pics.
    $if _xtal = 80
    T0CON = %00000000           '/ div2 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    TMR0H = 0                   '/ Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        '/ Start the Timer0.
$endif  '/ ScanTimer0
'-------------------------------------------------------------------------------
'/ Code to configure the Timer2.
$ifdef ScanTimer2
    IPR1Bits_TMR2IP = 1         '/ Timer2 for High Interrupt.

    $if _xtal = 4
        ' TMR2 8Bits counter, Postscaler 1/10, Prescaler 1/4, Clock Xtal/4 (Xtal: Xtal 4 Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01001001
        PR2 = 248
    $endif

    '/ used For Proteus
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/3, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 6,0 mS 
            T2CON = %00010011
            PR2 = 249
        $endif
    $else
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/5, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 8,0 mS 
            T2CON = %00100011
            PR2 = 249
        $endif
    $endif

    $if _xtal = 16
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 4 + PLL = 16 Mhz) or (Xtal: Xtal 16Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01111011
        PR2 = 155        
    $endif
    $if _xtal = 20
        ' TMR2 8Bits counter, Postscaler 1/13, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 10,00 mS 
        T2CON = %01100011
        PR2 = 239
    $endif
    $if _xtal = 32
        ' TMR2 8Bits counter, Postscaler 1/4, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 8,2 mS 
        T2CON = %01111011
        PR2 = 255
    $endif
    $if _xtal = 40
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz) or (Xtal: Xtal 10 Mhz + PLL)
        ' For Timer2 interrupt every 6,6 mS 
        T2CON = %01111011
        PR2 = 255
    $endif
    $if _xtal = 64
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 16 + PLL = 64Mhz)
        ' For Timer2 interrupt every 4,2 mS
        T2CON = %01111011
        PR2 = 255
    $endif

    ' Updated for Xtal = 80 Mhz, but for testing only because the Timing is only 3,2 mS.
    ' For K20 series.
    $if _xtal = 80
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 20 + PLL = 80Mhz)
        ' For Timer2 interrupt every 3,3 mS
        T2CON = %01111011
        PR2 = 255
    $endif       
       
    TMR2 = 0                    '/ Clear the Timer2 counter.
    Nop    
    T2CONBits_TMR2ON = 1        '/ Start the Timer2.
$endif  '/ ScanTimer2
'-------------------------------------------------------------------------------
    '/ Set the PORTS & Pins to digital.
    ADCON1 = 7
	Output SEGMENTS             '/ Put the display PORT as Output.

    ' Blanks directly the Port at reset depending of the option.
    $if DisplayMode = 0
    SEGMENTS = 0                '/ All segments cleared for common cathode.
    $else
    SEGMENTS = 255              '/ All segments cleared for common anode.
    $endif

    ' Write the lines OFF at reset.
    High LE2        '/ All digits off.
    Low AddressDA
    Low AddressDB
    Low AddressDC

	TIMESHARE = 1		        '/ Reset the time share variable
    dDISP_Flags = 0 
    
    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    PRINT_TENTHDTHOUSANDS = 255     '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
    PRINT_ONES = 255            '/ Blank the ONES DIGIT
    PRINT_TENS = 255            '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    $endif

    Clear DispDummyArray

    '/ Set all parameters of the array to work.
    GoSub RestoreArrayParameters_Sub

'/ Code to configure the Timer0.
$ifdef ScanTimer0
    ' Configure the interrupts to start.
    INTCONBits_TMR0IF = 0   '/ Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1   '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
$endif

'/ Code to configure the Timer0.
$ifdef ScanTimer2
    ' Configure the interrupts to start.
    PIR1Bits_TMR2IF = 0     '/ Clear the Timer0 interrupt flag to begin.
    PIE1Bits_TMR2IE = 1     '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
$endif
    ' The Timer is running...
    Return
'===============================================================================
_MultiplexDigitsExit:
'===============================================================================
    DisplayInit()       '/ Initialize the Displays.
'===============================================================================
$endif  ' _MultiplexDigits
'===============================================================================





